function [file_list quicklook] = idra_download_data(start_date, end_date, directory, data_type)
% idra_download_data: Download processed IDRA data from the 3TU.Datacentre to the local disk
%
%   Input:  start_date  .. start date as MATLAB serial date number
%           end_date    .. stop date as MATLAB serial date number
%           directory   .. directory where to store the files, string variable
%           data_type   .. -1 - all processed data files
%                           0 - quicklook data file only
%                           1 - near-range processed data only (1.5km range @3m range resolution)
%                           2 - standard-range processed data only (15km range @30m range resolution)
%                           3 - far-range processed data only (60km range @30m range resolution)
%                               --> far-range does not work, as the processed files are split up <--
%
%   Output: file_list   .. cell array of strings containing the full path and filename of the
%                          downloaded IDRA NetCDF files
%           quicklook   .. a structure containing the quicklook <data> as uint16 and <time> axis,
%                          hint for the analysis of quicklook.data array, use:
%                          bitget(quicklook.data, 1)  -> processed IDRA data availability
%                          bitget(quicklook.data, 2)  -> near-range data classified as no rain
%                          bitget(quicklook.data, 3)  -> near-range data classified as weak rain
%                          bitget(quicklook.data, 4)  -> near-range data classified as moderate rain
%                          bitget(quicklook.data, 5)  -> near-range data classified as strong rain
%                          bitget(quicklook.data, 6)  -> standard-range data classified as no rain
%                          bitget(quicklook.data, 7)  -> standard-range data classified as weak rain
%                          bitget(quicklook.data, 8)  -> standard-range data classified as moderate rain
%                          bitget(quicklook.data, 9)  -> standard-range data classified as strong rain
%                          bitget(quicklook.data, 10) -> far-range data classified as no rain
%                          bitget(quicklook.data, 11) -> far-range data classified as weak rain
%                          bitget(quicklook.data, 12) -> far-range data classified as moderate rain
%                          bitget(quicklook.data, 13) -> far-range data classified as strong rain
%                          bitget(quicklook.data, 14) -> raw data availability
%
% Created by Tobias Otto (tobias.otto@gmx.de), December 2011.

file_list      = {};
quicklook.data = [];
quicklook.time = [];

if ~strcmp(directory(end), '\'), directory = [directory '\']; end

days = fix(start_date):1:fix(end_date);
for ii=1:length(days)
    yy = datestr(days(ii), 'yyyy');
    mm = datestr(days(ii), 'mm');
    dd = datestr(days(ii), 'dd');

    % first download the quicklook file
    filename = ['IDRA_' yy '-' mm '-' dd '_quicklook.nc'];
    filestr  = [directory filename];
    if isempty(dir(filestr))
        [filestr,status] = urlwrite(['http://opendap.tudelft.nl/thredds/fileServer/IDRA/' yy '/' mm '/' dd '/' filename], filestr);
        if status == 0, continue; end
    end
    
    % open and read the quicklook file
    ncid = netcdf.open([filestr], 'NC_NOWRITE');
    tmp  = netcdf.getVar(ncid, netcdf.inqVarID(ncid, 'quicklook'));
    quicklook.data = cat(1, quicklook.data, tmp);
    tmp  = netcdf.getVar(ncid, netcdf.inqVarID(ncid, 'time'));
    tmp  = days(ii) + tmp/24;
    quicklook.time = cat(1, quicklook.time, tmp);
    netcdf.close(ncid);
    
    % download the requested IDRA data file
    for aa=1:3
        if ((data_type ~= aa) && (data_type ~= -1)) || (data_type == 0), continue; end
        if aa == 1, addon = '_near_range.nc'; end
        if aa == 2, addon = '_standard_range.nc'; end
        if aa == 3, addon = '_far_range.nc'; end
        
        filename = ['IDRA_' yy '-' mm '-' dd addon];    
        filestr  = [directory filename];
        if isempty(dir(filestr))
            [filestr,status] = urlwrite(['http://opendap.tudelft.nl/thredds/fileServer/IDRA/' yy '/' mm '/' dd '/' filename], filestr);
            if status, file_list{end+1} = filestr; end
        else
            file_list{end+1} = filestr;
        end
    end
end

quicklook.data = uint16(quicklook.data);

end

