function [variables standard_dir plot_ranges plot_titles plot_names file_format] = idra_plot_constants
% idra_plot_constants
%   Constants such as plot title, filename, colourbar-range for all available IDRA variables.
%   These constants are used by the MATLAB routine <idra_plot_ppi.m>.
%   You might consider changing the variables <plot_ranges>, <plot_titles> and <plot_names> if you
%   wish.
%
% Outputs:  variables    .. variable names for the dialog box
%           standard_dir .. directory to store plots offered as selection in the corresponding dialog box
%           plot_ranges  .. colourbar-range
%           plot_titles  .. plot titles
%           plot_names   .. plot filenames
%           file_format  .. desired output format of the figures
%
% Created by Tobias Otto (tobias.otto@gmx.de), November 2012.

% <-------------------------- Please change values here if desired -------------------------->

% desired output format (if '.fig' is selected, the figures are additionally also stored in '.png' format)
file_format = '.pdf';

% directory to store plots offered as standard selection in the corresponding dialog box
standard_dir = 'D:\';

% colourbar-range for each variable
range_z             = [-40 50];    % reflectivity (dBZ)
range_zdr           = [-2 5];      % differential reflectivity (dB)
range_ldr           = [-35 10];    % linear depolarisation ratio (dB)
range_vel           = [-20 20];    % Doppler velocity (ms-1)
range_wid           = [0 5];       % Doppler spectrum width (ms-1)
range_pdp           = [-10,40];    % two-way differential propagation phase (deg)
range_kdp           = [-2,10];     % one-way specific differential phase (deg km-1)
range_kdp_std       = [-0.06,3];   % standard deviation of the one-way specific differential phase (deg km-1)
range_dco           = [-5,15];     % differential backscatter phase (deg)
range_dco_std       = [-0.07,4];   % standard deviation of the differential backscatter phase (deg)
range_rainrate_std  = [0,30];      % standard deviation of the rainfall rate (mm h-1)

% the colormap used for the instantaneous rainfall rate is adapted for a plotting range of [-2 100]
% if you change this interval, consider also to change the colorbar (variable cmap) stored in the
% MATLAB file <idra_rainrate_colormap.mat>
range_rainrate      = [-1.5873,100];  % instantaneous rainfall rate (mm h-1)
range_flags         = [0, 8];         % range for the dataset flags, don't change

% plot titles
title_z             = 'z_{hh}^{att} (dBZ)';             % measured (attenuated) reflectivity (dBZ)
title_z_corr        = 'z_{hh} (dBZ)';                   % attenuation-corrected reflectivity (dBZ)
title_zdr           = 'z_{dr}^{att} (dB)';              % measured (attenuated) differential reflectivity (dB)
title_ldr           = 'LDR (dB)';                       % linear depolarisation ratio (dB)
title_vel           = 'v_{hh} (ms^{-1})';               % Doppler velocity (ms-1)
title_wid           = 'w_{hh} (ms^{-1})';               % Doppler spectrum width (ms-1)
title_pdp           = '\Psi_{dp} (deg)';                % two-way differential propagation phase (deg)
title_kdp           = 'K_{dp} (deg km^{-1})';           % one-way specific differential phase (deg km-1)
title_kdp_std       = '\sigma_{KDP} (deg km^{-1})';     % standard deviation of the one-way specific differential phase (deg km-1)
title_dco           = '\delta_{co} (deg)';              % differential backscatter phase (deg)
title_dco_std       = '\sigma_{\delta} (deg)';          % standard deviation of the differential backscatter phase (deg)
title_rainrate      = 'rainfall rate (mm/h)';           % instantaneous rainfall rate (mm h-1)
title_rainrate_std  = '\sigma_{rainrate} (mm/h)';       % standard deviation of the rainfall rate (mm h-1)
title_flags         = 'dataset flags';                  % dataset flags

% plot filenames
name_z             = 'zhh_att';              % measured (attenuated) reflectivity (dBZ)
name_z_corr        = 'zhh_corr';             % attenuation-corrected reflectivity (dBZ)
name_zdr           = 'zdr_att';              % differential reflectivity (dB)
name_ldr           = 'ldr';                  % linear depolarisation ratio (dB)
name_vel           = 'vhh';                  % Doppler velocity (ms-1)
name_wid           = 'whh';                  % Doppler spectrum width (ms-1)
name_pdp           = 'psi_dp';               % two-way differential propagation phase (deg)
name_kdp           = 'kdp';                  % one-way specific differential phase (deg km-1)
name_kdp_std       = 'kdp_sd';               % standard deviation of the one-way specific differential phase (deg km-1)
name_dco           = 'dco';                  % differential backscatter phase (deg)
name_dco_std       = 'dco_sd';               % standard deviation of the differential backscatter phase (deg)
name_rainrate      = 'rainfall_rate';        % instantaneous rainfall rate (mm h-1)
name_rainrate_std  = 'rainfall_rate_sd';     % standard deviation of the rainfall rate (mm h-1)
name_flags         = 'flags';                % dataset flags


% <-------------------------- Do not change anything from this point on -------------------------->

% variable names for the dialog box (DON'T CHANGE THIS)
variables = {'IDRA observables:' '' 'attenuated reflectivity Z_hh (dBZ)' 'attenuated differential reflectivity Z_dr (dB)' ...
             'attenuated linear depolarisation ratio LDR (dB)' 'differential phase Psi_dp (deg)'   ...
             'Doppler velocity v_hh (ms-1)' 'Doppler spectrum width w_hh (ms-1)' '' '-------------------------------------------------------' ...
             '' 'IDRA post-processed observables:' '' 'att. corrected reflectivity Z_hh (dBZ)'     ...
             'specific differential phase K_dp (deg km-1)' 'standard deviation of K_dp (deg km-1)' ...
             'differential backscatter phase delta_co (deg)' 'standard deviation of delta_co (deg)'...
             'estimated rainfall rate (mmh-1)' 'standard deviation of the rainfall rate (mmh-1)'   ...
             'dataset flags'};

empty = zeros(1,2);
plot_ranges = cat(3, empty, empty, range_z, range_zdr, range_ldr, range_pdp, range_vel, range_wid, ...
                     empty, empty, empty, empty, empty, range_z, range_kdp, range_kdp_std,         ...
                     range_dco, range_dco_std, range_rainrate, range_rainrate_std, range_flags);

empty = '';
plot_titles = {empty; empty; title_z; title_zdr; title_ldr; title_pdp; title_vel; title_wid; ...
               empty; empty; empty; empty; empty; title_z_corr; title_kdp; title_kdp_std;    ...
               title_dco; title_dco_std; title_rainrate; title_rainrate_std; title_flags};

plot_names  = {empty; empty; name_z; name_zdr; name_ldr; name_pdp; name_vel; name_wid; ...
               empty; empty; empty; empty; empty; name_z_corr; name_kdp; name_kdp_std;    ...
               name_dco; name_dco_std; name_rainrate; name_rainrate_std; name_flags};

end

