function idra_plot_ppi(Pdata,Azimuth,Range,BinLength,Prange,Ptitle,Ppath,Pfile,varargin)
% idra_plot_ppi: Create coloured plan-position indicators (PPI) of radar data.
%
%   inputs:
%     Pdata     .. an three-dimensional matrix of the form [bin no., ray no., data no.],
%                  with <bin no.> the number of range bins per ray, <ray no.> the
%                  number of rays and <data no.> different types of data to
%                  be plotted, e.g. Zhh, Zdr, LDR etc.,
%                  or a two-dimensional matrix of the form [bin no., ray no.] to create only one PPI
%     Azimuth   .. an one-dimensional matrix of the form [ray no.] specifying
%                  the azimuth of each ray (radian) counted clockwise from north 0 to 2pi
%     Range     .. an one-dimensional matrix of the form [bin no.]
%                  specifying the range of the leading edge of each bin (meter)
%     BinLength .. radial length of the bins (meter)
%     Prange    .. plotting range for Pdata, an matrix of the form [Pmin Pmax, data no.],
%                  set to [ -1 -1,data no.] to use the full range of <Pdata(:,:,data no.)>
%     Ptitle    .. a cell string matrix containing the titles for the plot, an
%                  matrix of the form [data no., title string],
%                  or for <data_no> = 1 a simple char matrix
%     Ppath     .. string containing the full path of the location for saving the plots
%     Pfile     .. file name where the image is going to be saved, '' if no output file
%                  should be created, a cell string matrix of the form [data no., file string],
%                  or for <data_no> = 1 a simple char matrix
%     varargin  .. the data no. that indicates a plot of the rainfall rate, results in the use of a
%                  different colourbar
%
%   Created by Tobias Otto (tobias.otto@gmx.de), February 2009.

% check of the input arguments
Ndata    = size(Pdata,3);
Nray     = fix(size(Pdata,2));
Nbin     = size(Pdata,1);
NetCDF   = 0;

if (length(Range) ~= Nbin),        error('idra_plot_ppi:Input_Error','Check the dimensions of <Pdata> and <Range>.');          end
if (Ndata > 1) && ~iscell(Ptitle), error('idra_plot_ppi:Input_Error','<Ptitle> must be a cell array with one entry per PPI.'); end
if (Ndata > 1) && ~iscell(Pfile),  error('idra_plot_ppi:Input_Error','<Pfile> must be a cell array with one entry per PPI.');  end

optargin = size(varargin,2);
rain     = -1;
flags    = -1;
for ii=1:optargin,
    tmp   = varargin{1};
    rain  = tmp(1);
    flags = tmp(2);
end


% map the polar to cartesian coordinates for the plot
d=(-Nbin*BinLength+BinLength:BinLength:Nbin*BinLength)/1e3; % (km)
if (Nbin > 512) % for long-range scans, plot only a subset of the bins
    select_idx = ~logical(mod(1:Nbin,round(Nbin/512)));
    Nbin       = sum(select_idx(:));
    d          = d(cat(2,select_idx,select_idx));
    Pdata      = Pdata(select_idx, :, :);
end
DataCart = NaN * ones(Ndata, 2*Nbin, 2*Nbin);

sector    = 2*pi/Nray;
subsector = sector/1.5e-3;
DataCart  = nan(Ndata, 2*Nbin, 2*Nbin);
for rr=1:Nray
    for bb=1:Nbin
        for kk=1:subsector
            cosine = round(Nbin+ (bb-1)*cos(Azimuth(rr)+sector*(kk-1)/subsector));
            sine   = round(Nbin+ (bb-1)*sin(Azimuth(rr)+sector*(kk-1)/subsector));
            DataCart(:,cosine,sine)=Pdata(bb,rr,:);
        end
    end
end

% create and save the plots
for dd=1:Ndata
    if iscell(Ptitle), Ttitle = char(Ptitle(dd,:)); else Ttitle = Ptitle; end
    if iscell(Pfile),  file   = char(Pfile(dd,:));  else file   = Pfile;  end

    if strcmp(file(end-2:end),'fig')
        fig_hand = figure('name', Ttitle, 'NumberTitle','off', 'Visible','on');
    else
        fig_hand = figure('name', Ttitle, 'NumberTitle','off', 'Visible','off');
    end
    ThePlot  = zeros(2*Nbin, 2*Nbin);
    ThePlot(:,:) = DataCart(dd,:,:);

    if (Prange(1,1,dd)==Prange(1,2,dd)) && (Prange(1,1,dd)==-1)
        imagesc(d,d,ThePlot);
    else
        clims = Prange(:,:,dd);
        imagesc(d,d,ThePlot,clims);
    end

    set(gca, 'Ydir', 'normal')
    ch = colorbar;
    cmap=colormap;
    cmap(1, :)=[1 1 1];
    colormap(cmap);
    
    if (rain == dd)
        load('idra_rainrate_colormap');
        colormap(cmap);
    end

    if (flags == dd)
        load('idra_flags_colormap');
        colormap(cmap);
        set(ch, 'YTick', [0 1 2 4 8], 'YTickLabel',{'', 'Z-R', 'KDP-R', 'Ext./Sat.', 'ML'});
    end
    axis image

   title(Ttitle, 'FontSize', 13);
   xlabel('W-E Distance from Radar (km)', 'FontSize', 12);
   ylabel('S-N Distance from Radar (km)', 'FontSize', 12);
   text(17,-18,'(IDRA / TU Delft)','FontSize',12);

    if ~strcmp(file,''),
        if strcmp(file(end-2:end),'fig')
            tmp_file = file;
            png_name = [file(1:end-4) '.png'];
            saveas(fig_hand, [Ppath tmp_file]);
            saveas(fig_hand, [Ppath png_name]);
        else
            saveas(fig_hand, [Ppath file]);
        end
    end
    close(fig_hand);
end

end