function [header, dimension, data] = idra_read_3TU_NetCDF(filename, time, varargin)
% idra_read_3TU_NetCDF: Read the processed IDRA data from the NetCDF files available at the 
%                       3TU.Datacentre, i.e. from raw data and processed data NetCDF files.
%
%   Inputs:  filename .. path and filename of the NetCDF file, i.e. 'C:\Data\IDRA_2009-04-27_near_range.nc'
%            time     .. -1 to read the whole dataset, or
%                        a vector [start_time end_time] with serial date numbers of a time slot to be read
%            varargin .. (optional), if present and set to '1', then only the header are read, and 
%                                    provided to the user by the structure array <header>
%
%   Outputs: header    .. a structure containing some header information of the dataset
%            dimension .. a structure containing the dimensions of the dataset, i.e. range and azimuth
%            data      .. a structure containing the weather radar observables read from the
%                        file, each field is a 2-dimensional matrix of the dimensions
%                        [<range_no>, <azim_no>]:
%                                zhh - co-polarised reflectivity, horizontal polarisation (dBZ)
%                                zdr - differential reflectivity (dB)
%                                ldr - linear depolarisation ratio (dB)
%                                vhh - Doppler radial velocity (m/s)
%                                whh - Doppler spectrum width (m/s)
%                                pdp - differential propagation phase (degrees)
%                        --> invalid data is marked as <NaN>
%
%   Example: [header dimension data] = sr_read_IDRA_3TU_NetCDF('D:\IDRA_2009-04-27_near_range.nc', [datenum('2009-04-27 06:15') datenum('2009-04-27 06:30')]);
%
%   Created by Tobias Otto (t.otto@tudelft.nl), August 2010.

rot_speed  = 1;             % rotational speed of the antenna (rpm)

% check the additional argument is present, i.e. whether only the header shall be read
optargin    = length(varargin);
header_only = 0;
for ii=1:optargin, if (ii == 1) && (varargin{1} == 1), header_only = 1; end; end

% open the file and read the content
try
    ncid = netcdf.open(filename, 'NC_NOWRITE');
catch
    ['idra_read_3TU_NetCDF: The file ' filename ' was not found.']
    header    = -1;
    dimension = -1;
    data      = -1;
    return;
end

% read some of the header information
header.title                = netcdf.getAtt(ncid,0,'title');
header.institution          = netcdf.getAtt(ncid,0,'organisationName_dataset');
tmp1                        = netcdf.getAtt(ncid,1,'date_start_of_data');
tmp2                        = netcdf.getAtt(ncid,1,'date_end_of_data');
header.start_time           = datenum([tmp1(1:10) ' ' tmp1(12:19)]);       % MATLAB serial date number
header.stop_time            = datenum([tmp2(1:10) ' ' tmp2(12:19)]);       % MATLAB serial date number
header.range_resolution 	= double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'range_resolution')));     % (m)
header.radiation_wavelength = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'radiation_wavelength')); % (m)
header.sweep_time           = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'sweep_time'));           % (s)
header.frequency_excursion  = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'frequency_excursion'));  % (Hz)
header.tx_power             = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'tx_power'));             % (W)
header.sample_size          = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'sample_size'));
header.beam_width           = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'beam_width'));           % (rad)
header.elevation_angle      = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'elevation_angle'));      % (rad)

if header_only
    data      = -1;
    dimension = -1;
    netcdf.close(ncid);
    return;
end

% check whether it is a raw dataset or a processed one
if isempty(strfind(filename, 'raw')), raw_data = 0; else raw_data = 1; end

% read the dimensions
if ~raw_data
    dimension.time    = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'time')); 
    dimension.azimuth = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'azimuth')); 
else
    dimension.time    = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'time_processed_data')); 
    dimension.azimuth = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'azimuth_processed_data')); 
end
dimension.range = double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'range')));
dimension.time  = fix(header.start_time) + (dimension.time ./ 24);

% compute some further header information
header.range_no   = length(dimension.range);

% read the data
if (time(1) == -1)
    data.zhh = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'equivalent_reflectivity_factor')); 
    data.zdr = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'differential_reflectivity'));
    data.ldr = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'linear_depolarisation_ratio'));
    data.vhh = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'radial_velocity'));
    data.whh = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'spectrum_width'));
    data.pdp = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'differential_phase'));
else
    min_idx = find(dimension.time(:) >= time(1), 1);
    max_idx = find(dimension.time(:) <= time(2), 1, 'last');

    if isempty(min_idx) || isempty(max_idx) || (min_idx == max_idx)
        disp('sr_read_IDRA_3TU_NetCDF: The specificied <time> interval is out of range for the chosen dataset.');
        data      = -1;
        netcdf.close(ncid);
        return;
    end
    count = max_idx - min_idx + 1;
    
    if count == 0, netcdf.close(ncid); return; end
    
    dimension.time    = dimension.time(min_idx:max_idx);
    dimension.azimuth = dimension.azimuth(min_idx:max_idx);

    data.zhh = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'equivalent_reflectivity_factor'),       [0 min_idx-1], [header.range_no count]); 
    data.zdr = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'differential_reflectivity'),            [0 min_idx-1], [header.range_no count]);
    data.ldr = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'linear_depolarisation_ratio'),          [0 min_idx-1], [header.range_no count]);
    data.vhh = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'radial_velocity'),                      [0 min_idx-1], [header.range_no count]);
    data.whh = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'spectrum_width'),                       [0 min_idx-1], [header.range_no count]);
    data.pdp = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'differential_phase'),                   [0 min_idx-1], [header.range_no count]);
end

if (sum(data.zhh(:) == -999) > 0), data.zhh(data.zhh == -999) = NaN; end
if (sum(data.zdr(:) == -999) > 0), data.zdr(data.zdr == -999) = NaN; end
if (sum(data.ldr(:) == -999) > 0), data.ldr(data.ldr == -999) = NaN; end
if (sum(data.vhh(:) == -999) > 0), data.vhh(data.vhh == -999) = NaN; data.vhh(isnan(data.zhh)) = NaN; end
if (sum(data.whh(:) == -999) > 0), data.whh(data.whh == -999) = NaN; end
if (sum(data.pdp(:) == -999) > 0), data.pdp(data.pdp == -999) = NaN; data.pdp(isnan(data.zhh)) = NaN; end

header.start_time = dimension.time(1);
header.stop_time  = dimension.time(end);
header.azimuth_no = length(dimension.time);
rays_per_round    = round(60 / (rot_speed * 2 * double(header.sample_size) * header.sweep_time*1e-6));
header.subscan_no = round(header.azimuth_no/rays_per_round);

% transform the differential phase from radian to degree
data.pdp = 180/pi * data.pdp;

netcdf.close(ncid);

end