function [header, dimension, data, quicklook] = idra_read_products_NetCDF(filename, time, vars)
% idra_read_products_NetCDF: Read the post-processed IDRA data (rainfall rate, attenuation-corrected
%                            reflectivity and differential propagation and backscatter phase) from
%                            the NetCDF files created by the MATLAB function <idra_start_rainfall_rate_estimation.m>
%                            and available at the CESAR database.
%
%   Inputs:  filename .. path and filename of the NetCDF file, i.e. 'C:\Data\IDRA_2009-04-27_near_range.nc'
%            time     .. -1 or [] to read the whole dataset, or
%                        a vector [start_time end_time] with serial date numbers of a time slot to be read
%            vars     .. [1D array of length eight] vector of zero/one indicating the variables to
%                        be read, [zhh kdp kdp_std dco dco_std rainrate rainrate_std flags],
%                        e.g. [1 1 0 0 0 1 0 0] to read <zhh>, <kdp> and <rainrate>
%
%   Outputs: header    .. a structure containing some header information of the dataset
%            dimension .. a structure containing the dimensions of the dataset, i.e. range, azimuth and time
%            data      .. a structure containing the post-processed IDRA data,
%                         each field is a 2-dimensional array of the dimensions [<range_no>, <azim_no>]:
%                            zhh          - co-polarised reflectivity, horizontal polarisation corrected for
%                                           rain and gaseous attenuation(dBZ)
%                            kdp          - estimated specific differential phase (deg/km)
%                            kdp_std      - standard deviation of <kdp> (deg/km)
%                            dco          - estimated differential backscatter phase (deg)
%                            dco_std      - standard deviation <dco> (deg)
%                            rainrate     - estimated rainfall rate (mm/h)
%                            rainrate_std - standard deviation of <rainrate> (mm/h)
%                            flags        - dataset flags for range-bins with no rainfall estimation
%                                           due to total signal extinction / receiver saturation or
%                                           possible melting layer contamination; and for range-bins
%                                           with rainfall estimation, whether <rainrate> is based on
%                                           <kdp> or <zhh>; use the information in data.flags as:
%                                mask = bitget(data.flags, 1) - logical mask of rainfall estimates based on <zhh>
%                                mask = bitget(data.flags, 2) - logical mask of rainfall estimates based on <kdp>
%                                mask = bitget(data.flags, 3) - logical mask for total signal extinction / receiver saturation
%                                mask = bitget(data.flags, 4) - logical mask for possible melting layer contamination
%            quicklook .. a structure containing the information of daily rainfall accumulation displayed 
%                         in the quicklook, contains the fields
%                            ql_azimuth        - azimuth (rad) used for the PPI of the daily rainfall accumulation
%                            rain_accumulation - 2-dimensional array of the dimensions [<range_no length(quicklook.azimuth)]
%                                                with the daily rainfall accumulation (mm) based on <rainrate>
%                        --> invalid data is marked as <NaN>
%
%   Example: [header dimension data quicklook] = sr_read_IDRA_3TU_NetCDF('D:\cesar_idra_products_lb1_t00_v1.0_20101001.nc', [datenum('2010-11-10 06:15') datenum('2010-11-10 08:15')]);
%
%   Created by Tobias Otto (t.otto@tudelft.nl), November 2012.

rot_speed = 1;             % rotational speed of the IDRA antenna (rpm)

header    = [];
dimension = [];
data      = [];
quicklook = [];

if length(vars) ~= 8
    disp('idra_read_products_NetCDF: <vars> must be a vector with eight elements.');
    return;
end

% open the file and read the content
try
    ncid = netcdf.open(filename, 'NC_NOWRITE');
catch
    disp(['idra_read_products_NetCDF: The file ' filename ' was not found.']);
    return;
end

if isempty(time), time = -1; end

% read some of the header information
header.title                = netcdf.getAtt(ncid,0,'title');
header.institution          = netcdf.getAtt(ncid,0,'organisationName_dataset');
tmp1                        = netcdf.getAtt(ncid,1,'date_start_of_data');
tmp2                        = netcdf.getAtt(ncid,1,'date_end_of_data');
header.start_time           = datenum([tmp1(1:10) ' ' tmp1(12:19)]);       % MATLAB serial date number
header.stop_time            = datenum([tmp2(1:10) ' ' tmp2(12:19)]);       % MATLAB serial date number
header.range_resolution 	= double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'range_resolution')));     % (m)
tmp                         = netcdf.getAtt(ncid,netcdf.inqVarID(ncid,'product'),'radar_radiation_wavelength');
header.radiation_wavelength = str2num(tmp(end-5:end));                                          % (m)
tmp                         = netcdf.getAtt(ncid,netcdf.inqVarID(ncid,'product'),'sweep_time');
tmp_idx                     = regexp(tmp,':')+1;
header.sweep_time           = str2num(tmp(tmp_idx:end));                                        % (s)
header.frequency_excursion  = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'frequency_excursion'));  % (Hz)
tmp                         = netcdf.getAtt(ncid,netcdf.inqVarID(ncid,'product'),'sample_size');
tmp_idx                     = regexp(tmp,':')+1;
header.sample_size          = str2num(tmp(tmp_idx:end));
tmp                         = netcdf.getAtt(ncid,netcdf.inqVarID(ncid,'product'),'antenna_beam_width');
tmp_idx                     = regexp(tmp,':')+1;
header.antenna_beam_width   = str2num(tmp(tmp_idx:end));                                        % (deg)
tmp                         = netcdf.getAtt(ncid,netcdf.inqVarID(ncid,'product'),'elevation_above_horizon');
tmp_idx                     = regexp(tmp,':')+1;
header.antenna_elevation    = str2num(tmp(tmp_idx:end));                                        % (deg)
header.gaseous_attenuation  = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'gaseous_attenuation'));  % (dB)
header.diff_phase_offset    = netcdf.getVar(ncid,netcdf.inqVarID(ncid,'differential_phase_offset'));
header.diff_phase_offset(header.diff_phase_offset == -999) = nan;
header.diff_phase_offset    = header.diff_phase_offset * 180/pi;                                % (deg)


% read the dimensions
dimension.time    = double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'time'))); 
dimension.azimuth = double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'azimuth')));
dimension.range   = double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'range')));
dimension.time    = fix(header.start_time) + (dimension.time ./ 24);

% compute some further header information
header.range_no   = length(dimension.range);
header.azimuth_no = length(dimension.azimuth);

% read the data
if (time(1) ~= -1)
    min_idx = find(dimension.time(:) >= time(1), 1);
    max_idx = find(dimension.time(:) <= time(2), 1, 'last');

    if isempty(min_idx) || isempty(max_idx) || (min_idx == max_idx)
        disp('idra_read_products_NetCDF: The specificied <time> interval is out of range for the chosen dataset.');
        data      = -1;
        netcdf.close(ncid);
        return;
    end

    count = max_idx - min_idx + 1;
    dimension.time    = dimension.time(min_idx:max_idx);
    dimension.azimuth = dimension.azimuth(min_idx:max_idx);
end


if vars(1)
    id_zhh = netcdf.inqVarID(ncid,'equivalent_reflectivity_factor');
    if time(1) == -1, data.zhh = double(netcdf.getVar(ncid, id_zhh)); else ...
                      data.zhh = double(netcdf.getVar(ncid, id_zhh, [0 min_idx-1], [header.range_no count])); end
    scale_factor_zhh  = netcdf.getAtt(ncid,id_zhh,'scale_factor');
    fill_value_zhh    = netcdf.getAtt(ncid,id_zhh,'_FillValue');
    data.zhh(data.zhh == fill_value_zhh) = NaN;
    data.zhh          = data.zhh * scale_factor_zhh;
end

if vars(2)
    id_kdp = netcdf.inqVarID(ncid,'specific_differential_phase');
    if time(1) == -1, data.kdp = double(netcdf.getVar(ncid, id_kdp)); else ...
                      data.kdp = double(netcdf.getVar(ncid, id_kdp, [0 min_idx-1], [header.range_no count])); end
    scale_factor_kdp  = netcdf.getAtt(ncid,id_kdp,'scale_factor');
    fill_value_kdp    = netcdf.getAtt(ncid,id_kdp,'_FillValue');
    data.kdp(data.kdp == fill_value_kdp) = NaN;
    data.kdp          = data.kdp * scale_factor_kdp;
end

if vars(3)
    id_kdp_std = netcdf.inqVarID(ncid,'sigma_specific_differential_phase');
    if time(1) == -1, data.kdp_std = double(netcdf.getVar(ncid, id_kdp_std)); else ...
                      data.kdp_std = double(netcdf.getVar(ncid, id_kdp_std, [0 min_idx-1], [header.range_no count])); end
    scale_factor_kdp_std  = netcdf.getAtt(ncid,id_kdp_std,'scale_factor');
    fill_value_kdp_std    = netcdf.getAtt(ncid,id_kdp_std,'_FillValue');
    data.kdp_std(data.kdp_std == fill_value_kdp_std) = NaN;
    data.kdp_std          = data.kdp_std * scale_factor_kdp_std;
end

if vars(4)
    id_dco = netcdf.inqVarID(ncid,'differential_backscatter_phase');
    if time(1) == -1, data.dco = double(netcdf.getVar(ncid, id_dco)); else ...
                      data.dco = double(netcdf.getVar(ncid, id_dco, [0 min_idx-1], [header.range_no count])); end
    scale_factor_dco  = netcdf.getAtt(ncid,id_dco,'scale_factor');
    fill_value_dco    = netcdf.getAtt(ncid,id_dco,'_FillValue');
    data.dco(data.dco == fill_value_dco) = NaN;
    data.dco          = data.dco * scale_factor_dco;
end

if vars(5)
    id_dco_std = netcdf.inqVarID(ncid,'sigma_differential_backscatter_phase');
    if time(1) == -1, data.dco_std = double(netcdf.getVar(ncid, id_dco_std)); else ...
                      data.dco_std = double(netcdf.getVar(ncid, id_dco_std, [0 min_idx-1], [header.range_no count])); end
    scale_factor_dco_std  = netcdf.getAtt(ncid,id_dco_std,'scale_factor');
    fill_value_dco_std    = netcdf.getAtt(ncid,id_dco_std,'_FillValue');
    data.dco_std(data.dco_std == fill_value_dco_std) = NaN;
    data.dco_std          = data.dco_std * scale_factor_dco_std;
end

if vars(6)
    id_rainrate = netcdf.inqVarID(ncid,'rainfall_rate');
    if time(1) == -1, data.rainrate = double(netcdf.getVar(ncid, id_rainrate)); else ...
                      data.rainrate = double(netcdf.getVar(ncid, id_rainrate, [0 min_idx-1], [header.range_no count])); end
    fill_value_rainrate   = netcdf.getAtt(ncid,id_rainrate,'_FillValue');
    data.rainrate(data.rainrate == fill_value_rainrate) = NaN;
    
    scale_factor_rainrate = netcdf.getAtt(ncid,id_rainrate,'scale_factor');
    try
        offset_rainrate = netcdf.getAtt(ncid,id_rainrate,'add_offset');
        data.rainrate   = data.rainrate * scale_factor_rainrate + offset_rainrate;
    catch
        data.rainrate   = data.rainrate * scale_factor_rainrate;
    end
end

if vars(7)
    id_rainrate_std = netcdf.inqVarID(ncid,'sigma_rainfall_rate');
    if time(1) == -1, data.rainrate_std = double(netcdf.getVar(ncid, id_rainrate_std)); else ...
                      data.rainrate_std = double(netcdf.getVar(ncid, id_rainrate_std, [0 min_idx-1], [header.range_no count])); end
    fill_value_rainrate_std   = netcdf.getAtt(ncid,id_rainrate_std,'_FillValue');
    data.rainrate_std(data.rainrate_std == fill_value_rainrate_std) = NaN;
    
    scale_factor_rainrate_std = netcdf.getAtt(ncid,id_rainrate_std,'scale_factor');
    try
        offset_rainrate_std = netcdf.getAtt(ncid,id_rainrate_std,'add_offset');
        data.rainrate_std   = data.rainrate_std * scale_factor_rainrate_std + offset+rainrate_std;
    catch
        data.rainrate_std   = data.rainrate_std * scale_factor_rainrate_std;
    end
end

if vars(8)
    id_flags = netcdf.inqVarID(ncid,'dataset_flags');
    if time(1) == -1, data.flags = double(netcdf.getVar(ncid, id_flags)); else ...
                      data.flags = double(netcdf.getVar(ncid, id_flags, [0 min_idx-1], [header.range_no count])); end
end

% read the quicklook
quicklook.ql_azimuth        = double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'quicklook_azimuth')));
quicklook.rain_accumulation = double(netcdf.getVar(ncid,netcdf.inqVarID(ncid,'thickness_of_daily_rainfall_amount'))) * 1000;

netcdf.close(ncid);

end