function idra_plot_data
% idra_plot_data 
%   Plot PPI's of rainfall rate and polarimetric weather radar observables from IDRA NetCDF 
%   products file and/or processed data file.
%
%   The following NetCDF IDRA files are supported:
%     cesar_idra_products_lb1_t00_v1.0_yyyymmdd.nc  (online available at http://www.cesar-database.nl)
%     IDRA_yyyy-mm-dd_near_range.nc                 (online available at http://data.3tu.nl/repository/collection:cabauw)
%     IDRA_yyyy-mm-dd_standard_range.nc             (online available at http://data.3tu.nl/repository/collection:cabauw)
%     IDRA_yyyy-mm-dd_far_range.nc                  (online available at http://data.3tu.nl/repository/collection:cabauw)
%
%   Note:
%   Some parameters of the figures such as colourbar-range, filename, filetype and plot title can be
%   manually configured in the MATLAB routine <idra_plot_constants.m>
%
% Created by Tobias Otto (tobias.otto@gmx.de), November 2012.

% NetCDF file selection
[filename pathname] = uigetfile({'*standard_range.nc; *near_range.nc; *far_range.nc; *idra_products*.nc','Processed IDRA data in NetCDF format'});
if isequal(filename,0), return; end % user selected cancel

% check whether whether the user selected an IDRA products NetCDF file or only a processed data file
plot_products = logical(strfind(filename,'idra_products'));
if isempty(plot_products)
    plot_products  = 0; 
    products_file  = [];
    processed_file = [pathname filename];
else
    products_file = [pathname filename];
end

% get some constants used for the dialog boxes and plots (title, colourbar-range, filenames)
[variables standard_dir plot_ranges plot_titles plot_names file_format] = idra_plot_constants;

% selection of the IDRA observables to be plotted
if ~plot_products, variables(9:end) = []; end

[s,v] = listdlg('Name','Variable Selection', 'PromptString','Select variables to be plotted:', ...
                'SelectionMode','multiple',   'ListString', variables, 'ListSize',[300 400],   ...
                'InitialValue',2);
if ~v, return; end                                          % user selected cancel
pause(1);
if sum((s>=3 & s<=8) | (s>=14 & s<=21)) == 0, return; end   % user selected no valid observable
s((s<3) | (s>8 & s<14)) = [];


% user selected products file, but wants to plot also processed data, check for the presence of the
% file, and download it in case
if plot_products && logical(sum(s < 9))
    [header, dimension, data, quicklook] = idra_read_products_NetCDF(products_file, -1, zeros(8,1));
    date_to_plot = fix(header.start_time);
    processed_file = idra_download_data(date_to_plot, date_to_plot, pathname, 2);
    processed_file = processed_file{1};
    clear('header','dimension','quicklook');
elseif ~plot_products
    [header, dimension, data] = idra_read_3TU_NetCDF(processed_file, -1, 1);
    date_to_plot = fix(header.start_time);
    clear('header','dimension');
else
    [header, dimension, data, quicklook] = idra_read_products_NetCDF(products_file, -1, zeros(8,1));
    date_to_plot = fix(header.start_time);
    clear('header','dimension','quicklook');
    processed_file = [];
end


% selection of the time span for the plotting of the data
tmp = 0;
def = {standard_dir,'00:00','23:59'};
while tmp==0
    prompt    = {'Directory to store the figures',['Start at ' datestr(date_to_plot) ' HH:MM'],['End at ' datestr(date_to_plot) ' HH:MM']};
    dlg_title = 'Select time span';
    num_lines = 1;
    answer = inputdlg(prompt,dlg_title,num_lines,def);
    pause(1);

    if isempty(answer), return; end

    % check the user inputs
    try
        plot_directory = answer{1};
        start_time     = datenum([datestr(date_to_plot) ' ' answer{2}]);
        end_time       = datenum([datestr(date_to_plot) ' ' answer{3}]);
        tmp            = 1;
    catch
        h = errordlg('idra_plot_data: Error in converting the dates, please check the format (HH:MM)!');
        uiwait(h);
        def = answer;
        tmp = 0;
    end
end

% create a sub-directory for the figures to be plotted
if ~strcmp(plot_directory(end),'\'), plot_directory(end+1) = '\'; end
plot_directory = [plot_directory 'idra_ppi_' datestr(fix(start_time), 'yyyy-mm-dd') '\'];
mkdir(plot_directory);

% start to plot the PPI's, hour by hour to prevent 'out of memory' error messages
start_hour = (fix(start_time*24)/24 - fix(start_time)) * 24;
cycle_no = ceil((end_time - start_time)*24) + start_hour;
if fix(end_time*24) ==  fix(start_time*24), cycle_no = start_hour; end
for hh=start_hour:cycle_no
    to_plot   = s;
    orig_vars = [];
    proc_vars = [];
    tmp_start = date_to_plot + hh/24;
    tmp_stop  = date_to_plot + (hh+1)/24;
    if (hh == start_hour), tmp_start = start_time; end
    if (hh == cycle_no),   tmp_stop  = end_time; end
    if (tmp_stop <= tmp_start), continue; end

    % load the data for the current hour
    if ~isempty(processed_file) && (sum(s>=3 & s<=8) > 0)    % processed data selected for plotting
        [orig_header, orig_dimension, orig_data] = idra_read_3TU_NetCDF(processed_file, [tmp_start tmp_stop]);
        if isstruct(orig_data)      % extract the variables to plot
            orig_vars = zeros(orig_header.range_no, length(orig_dimension.time), sum(s>=3 & s<=8));
            ii = 1;
            if sum(s == 3), orig_vars(:,:,ii) = orig_data.zhh; ii = ii+1; end
            if sum(s == 4), orig_vars(:,:,ii) = orig_data.zdr; ii = ii+1; end
            if sum(s == 5), orig_vars(:,:,ii) = orig_data.ldr; ii = ii+1; end
            if sum(s == 6), orig_vars(:,:,ii) = orig_data.pdp; ii = ii+1; end
            if sum(s == 7), orig_vars(:,:,ii) = orig_data.vhh; ii = ii+1; end
            if sum(s == 8), orig_vars(:,:,ii) = orig_data.whh; ii = ii+1; end
            clear('orig_data');
        end
    end

    if ~isempty(products_file) && (sum(s>=14 & s<=21) > 0)   % post-processed data selected for plotting
        tmp  = s; tmp(tmp < 14) = []; tmp = tmp - 13;
        tmp2 = false(8,1); tmp2(tmp) = true;
        [proc_header, proc_dimension, proc_data, proc_quicklook] = idra_read_products_NetCDF(products_file, [tmp_start tmp_stop], tmp2);
        % post-processed data is only available for regions with rain, check whether there was any rain in the current hour
        if ~isstruct(proc_data)
            to_plot(to_plot >= 14) = [];
        else
            proc_vars = zeros(proc_header.range_no, length(proc_dimension.time), sum(s>=14 & s<=21));
            ii = 1;
            if sum(s == 14), proc_vars(:,:,ii) = proc_data.zhh;     ii = ii+1; end
            if sum(s == 15), proc_vars(:,:,ii) = proc_data.kdp;     ii = ii+1; end
            if sum(s == 16), proc_vars(:,:,ii) = proc_data.kdp_std; ii = ii+1; end
            if sum(s == 17), proc_vars(:,:,ii) = proc_data.dco;     ii = ii+1; end
            if sum(s == 18), proc_vars(:,:,ii) = proc_data.dco_std; ii = ii+1; end
            if sum(s == 19), proc_vars(:,:,ii) = proc_data.rainrate;     ii = ii+1; end
            if sum(s == 20), proc_vars(:,:,ii) = proc_data.rainrate_std; ii = ii+1; end
            if sum(s == 21), proc_vars(:,:,ii) = proc_data.flags;   ii = ii+1; end
            clear('proc_data');
        end
    end

    % continue if in the current hour there is no data to be plotted
    if isempty(to_plot) || (sum(s>=3 & s<=8) == length(to_plot) && isempty(orig_vars)), continue; end
    
    if ~isempty(proc_vars) && ~isempty(orig_vars)
        proc_tmp       = nan(size(orig_vars,1), size(orig_vars,2), size(proc_vars,3));
        tmp_idx        = nan(1, length(proc_dimension.time));
        for ii=1:length(proc_dimension.time), [bla tmp_idx(ii)] = min(abs(orig_dimension.time(:) - proc_dimension.time(ii))); end
        proc_tmp(:,tmp_idx,:) = proc_vars;
        plot_vars      = cat(3, orig_vars, proc_tmp);
        plot_time      = orig_dimension.time;
        plot_azim      = orig_dimension.azimuth;
        plot_range     = orig_dimension.range;
        plot_range_res = orig_header.range_resolution;
        clear('proc_tmp');
    elseif ~isempty(proc_vars)
        plot_vars      = proc_vars;
        plot_time      = proc_dimension.time;
        plot_azim      = proc_dimension.azimuth;
        plot_range     = proc_dimension.range;
        plot_range_res = proc_header.range_resolution;
    elseif ~isempty(orig_vars)
        plot_vars      = orig_vars;
        plot_time      = orig_dimension.time;
        plot_azim      = orig_dimension.azimuth;
        plot_range     = orig_dimension.range;
        plot_range_res = orig_header.range_resolution;
    end
    clear('proc_vars', 'orig_vars');

    % finally call the plot routine minute-for-minute, i.e. PPI per PPI for IDRA's 1rpm antenna speed
    no_of_minutes = ceil((tmp_stop - tmp_start)*24*60);
    for mm=1:no_of_minutes
        start_min = tmp_start + (mm-1)/60/24;
        stop_min  = tmp_start + mm/60/24;
        disp(['Currently plotting ' datestr(start_min, 'yyyy-mm-dd HH:MM')]);

        % plot all data falling in the current minute
        sub_idx = (plot_time >= start_min) & (plot_time < stop_min);
        if sum(sub_idx) == 0, continue; end

        tmp_titles    = plot_titles(to_plot);
        tmp_addon     = [datestr(start_min,'yyyy-mm-dd HH:MM') 'UTC - ' datestr(stop_min,'HH:MM') 'UTC    '];
        tmp_titles(:) = strcat({tmp_addon}, tmp_titles(:));

        tmp_names    = plot_names(to_plot);
        tmp_addon    = [datestr(start_min,'_yyyy-mm-dd_HHMM') file_format];
        tmp_names(:) = strcat(tmp_names(:), {tmp_addon});

        varargin = [-1 -1];
        if sum(to_plot == 19), varargin(1) = find(to_plot == 19); end
        if sum(to_plot == 21), varargin(2) = find(to_plot == 21); end
        idra_plot_ppi(plot_vars(:,sub_idx,:), plot_azim(sub_idx), plot_range, plot_range_res, ...
                      plot_ranges(:,:,to_plot), tmp_titles, plot_directory, tmp_names, varargin);
    end
    clear('plot_vars');
end

clear all;
close all;

end